/*****************************************************************************
 * Copyright (c) 2014, 2017 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   CEA LIST - Initial API and implementation
 *****************************************************************************/
package org.eclipse.papyrus.cdo.internal.core.resource;

import java.io.File;

import org.eclipse.emf.cdo.explorer.CDOExplorerUtil;
import org.eclipse.emf.cdo.explorer.checkouts.CDOCheckout;
import org.eclipse.emf.cdo.util.CDOURIUtil;
import org.eclipse.emf.common.util.URI;
import org.eclipse.papyrus.cdo.internal.core.Activator;
import org.eclipse.papyrus.infra.core.resource.ModelSet;
import org.eclipse.papyrus.infra.core.resource.sasheditor.AbstractSashModelProvider;
import org.eclipse.papyrus.infra.core.resource.sasheditor.SashModel;


/**
 * A CDO-specific sash model storage provider.
 */
public class CDOSashModelProvider extends AbstractSashModelProvider {

	private static final String BASE_FOLDER_NAME = "sashidx"; // $NON-NLS-1$

	private File indexFolder;

	public CDOSashModelProvider() {
	}

	@Override
	public void initialize(ModelSet modelSet) {
		super.initialize(modelSet);

		CDOCheckout checkout = CDOExplorerUtil.getCheckout(modelSet);
		if (checkout != null) {
			indexFolder = checkout.getStateFolder(BASE_FOLDER_NAME, true);
		} else {
			indexFolder = new File(Activator.getDefault().getStateLocation().toFile(), BASE_FOLDER_NAME);
			Activator.log.debug("CDOSashModelProvider can not find a checkout for model set " + modelSet + ". Storing sash models in " + indexFolder);
		}
	}

	@Override
	public void dispose() {
		indexFolder = null;
		super.dispose();
	}

	@Override
	public URI getSashModelURI(URI userModelURI) {
		String relativePath = CDOURIUtil.extractResourcePath(userModelURI.trimFileExtension());
		String path = new File(indexFolder, relativePath).getAbsolutePath();
		return URI.createFileURI(path).appendFileExtension(SashModel.SASH_MODEL_FILE_EXTENSION);
	}
}
